#ITPcnv by flamethrower
#Converts PNG256 to ITP
import os
import struct
import warnings

import png
import falcom_compress

def ITPcnv(infilename, outfilename = None):
    '''Convert PNG256 to ITP.

    Default output is input filename with .itp extension in same folder
    as input file.
    '''
    #Run pyPNG read
    w, h, pixels, attribs = png.Reader(infilename).read()

    #Check to make sure it's ok for this tool
    #Format must be 8 bit color depth, indexed color
    if attribs['bitdepth'] != 8:
        raise Exception('{}: This tool works with 8-bit color depth PNGs only'
                        .format(infilename))
    x = attribs.get('palette', None)
    if x == None:
        raise Exception('{}: This tool works with indexed-color PNGs only'
                        .format(infilename))

    #Check for transparency information
    if len(x[0]) != 4:
        warnings.warn('{}: Source image is missing transparency information.\n'
                      'Assuming all pixels are opaque.'.format(infilename))
        transparency_correction_flag = True
    else:
        transparency_correction_flag = False

    #Convert palette to bytes (from r, g, b, a tuples)
    pal = []
    if transparency_correction_flag:
        for color in attribs['palette']:
            pal.append(struct.pack('BBBB', *color, 255))
    else:    
        for color in attribs['palette']:
            pal.append(struct.pack('BBBB', *color))

    #Convert bitmap to bytes (from byte arrays)
    bitmap = []
    for row in pixels:
        bitmap.append(struct.pack('B' * w, *row))

    if outfilename == None:
        outfilename = (
            os.path.splitext(os.path.split(infilename)[1])[0] + '.itp')
        outfilename = os.path.join(os.path.split(infilename)[0], outfilename)

    #Output the ITP file
    with open(outfilename, 'wb') as f:
        f.write(struct.pack('<III', 0x3EA, w, h))
        f.write(falcom_compress.compress_FALCOM3(b''.join(pal)))
        f.write(falcom_compress.compress_FALCOM3(b''.join(bitmap)))
        f.write(b'\x00')

#Example call
##ITPcnv(r'..\charsel.png')
import os
for dirpath, dirnames, filenames in os.walk('.'):
    for filename in filenames:
        if os.path.splitext(filename)[1] == '.png':
            try:
                print(filename)
                ITPcnv(dirpath + '\\' + filename)
            except Exception:
                print('Failed: {}'.format(dirpath + '\\' + filename))
